@@ -6,7 +6,8 @@ from django.db.models.signals import post_save
 from django.dispatch import receiver
 from django.utils import timezone
 
-from .models import Circuit, CircuitTermination
+from .choices import CircuitTerminationSideChoices
+from .models import CircuitTermination
 from nautobot.dcim.models import CablePath
 from nautobot.dcim.signals import create_cablepath
 
@@ -32,22 +33,26 @@ def rebuild_paths_circuits(obj):
 
 
 @receiver(post_save, sender=CircuitTermination)
-def update_circuit(instance, **kwargs):
+def update_circuit(instance, raw=False, **kwargs):
     """
     When a CircuitTermination has been modified, update its parent Circuit.
     """
-    fields = {
-        "last_updated": timezone.now(),
-        f"termination_{instance.term_side.lower()}": instance.pk,
-    }
-    Circuit.objects.filter(pk=instance.circuit_id).update(**fields)
+    if raw:
+        return
+    if instance.term_side in CircuitTerminationSideChoices.values():
+        termination_name = f"termination_{instance.term_side.lower()}"
+        setattr(instance.circuit, termination_name, instance)
+        setattr(instance.circuit, "last_updated", timezone.now())
+        instance.circuit.save()
 
 
 @receiver(post_save, sender=CircuitTermination)
-def update_connected_terminations(instance, **kwargs):
+def update_connected_terminations(instance, raw=False, **kwargs):
     """
     When a CircuitTermination has been modified, update the Cable Paths if the Circuit Termination has a peer.
     """
+    if raw:
+        return
     peer = instance.get_peer_termination()
     # Check if Circuit Termination has a peer
     if peer:
