@@ -39,11 +39,11 @@ class CircuitTypeUIViewSet(
         # Circuits
         context = super().get_extra_context(request, instance)
         if self.action == "retrieve":
-            # v2 TODO(jathan): Replace prefetch_related with select_related
             circuits = (
                 Circuit.objects.restrict(request.user, "view")
                 .filter(type=instance)
-                .prefetch_related("type", "tenant", "terminations__site")
+                .select_related("type", "tenant")
+                .prefetch_related("terminations__site")
             )
 
             circuits_table = tables.CircuitTable(circuits)
@@ -93,11 +93,11 @@ class ProviderUIViewSet(NautobotUIViewSet):
     def get_extra_context(self, request, instance):
         context = super().get_extra_context(request, instance)
         if self.action == "retrieve":
-            # v2 TODO(jathan): Replace prefetch_related with select_related
             circuits = (
                 Circuit.objects.restrict(request.user, "view")
                 .filter(provider=instance)
-                .prefetch_related("type", "tenant", "terminations__site")
+                .select_related("type", "tenant")
+                .prefetch_related("terminations__site")
             )
             circuits_table = tables.CircuitTable(circuits)
             circuits_table.columns.hide("provider")
@@ -129,10 +129,9 @@ class CircuitUIViewSet(NautobotUIViewSet):
         context = super().get_extra_context(request, instance)
         if self.action == "retrieve":
             # A-side termination
-            # v2 TODO(jathan): Replace prefetch_related with select_related
             termination_a = (
                 CircuitTermination.objects.restrict(request.user, "view")
-                .prefetch_related("site__region")
+                .select_related("site__region")
                 .filter(circuit=instance, term_side=CircuitTerminationSideChoices.SIDE_A)
                 .first()
             )
@@ -146,10 +145,9 @@ class CircuitUIViewSet(NautobotUIViewSet):
                 )
 
             # Z-side termination
-            # v2 TODO(jathan): Replace prefetch_related with select_related
             termination_z = (
                 CircuitTermination.objects.restrict(request.user, "view")
-                .prefetch_related("site__region")
+                .select_related("site__region")
                 .filter(circuit=instance, term_side=CircuitTerminationSideChoices.SIDE_Z)
                 .first()
             )
@@ -181,11 +179,11 @@ class ProviderNetworkUIViewSet(NautobotUIViewSet):
     def get_extra_context(self, request, instance):
         context = super().get_extra_context(request, instance)
         if self.action == "retrieve":
-            # v2 TODO(jathan): Replace prefetch_related with select_related
             circuits = (
                 Circuit.objects.restrict(request.user, "view")
                 .filter(Q(termination_a__provider_network=instance.pk) | Q(termination_z__provider_network=instance.pk))
-                .prefetch_related("type", "tenant", "terminations__site")
+                .select_related("type", "tenant")
+                .prefetch_related("terminations__site")
             )
 
             circuits_table = tables.CircuitTable(circuits)
