@@ -75,7 +75,7 @@ from nautobot.dcim.models import (
 from nautobot.extras.api.serializers import (
     NautobotModelSerializer,
     StatusModelSerializerMixin,
-    TaggedObjectSerializer,
+    TaggedModelSerializerMixin,
 )
 from nautobot.extras.api.nested_serializers import NestedConfigContextSchemaSerializer, NestedSecretsGroupSerializer
 from nautobot.extras.models import Status
@@ -89,6 +89,7 @@ from nautobot.tenancy.api.nested_serializers import NestedTenantSerializer
 from nautobot.users.api.nested_serializers import NestedUserSerializer
 from nautobot.utilities.api import get_serializer_for_model
 from nautobot.utilities.config import get_settings_or_config
+from nautobot.utilities.deprecation import class_deprecated_in_favor_of
 from nautobot.virtualization.api.nested_serializers import NestedClusterSerializer
 
 # Not all of these variable(s) are not actually used anywhere in this file, but required for the
@@ -132,7 +133,7 @@ from .nested_serializers import (  # noqa: F401
 )
 
 
-class CableTerminationSerializer(serializers.ModelSerializer):
+class CableTerminationModelSerializerMixin(serializers.ModelSerializer):
     cable_peer_type = serializers.SerializerMethodField(read_only=True)
     cable_peer = serializers.SerializerMethodField(read_only=True)
 
@@ -154,7 +155,13 @@ class CableTerminationSerializer(serializers.ModelSerializer):
         return None
 
 
-class ConnectedEndpointSerializer(ValidatedModelSerializer):
+# TODO: remove in 2.2
+@class_deprecated_in_favor_of(CableTerminationModelSerializerMixin)
+class CableTerminationSerializer(CableTerminationModelSerializerMixin):
+    pass
+
+
+class PathEndpointModelSerializerMixin(ValidatedModelSerializer):
     connected_endpoint_type = serializers.SerializerMethodField(read_only=True)
     connected_endpoint = serializers.SerializerMethodField(read_only=True)
     connected_endpoint_reachable = serializers.SerializerMethodField(read_only=True)
@@ -183,6 +190,12 @@ class ConnectedEndpointSerializer(ValidatedModelSerializer):
         return None
 
 
+# TODO: remove in 2.2
+@class_deprecated_in_favor_of(PathEndpointModelSerializerMixin)
+class ConnectedEndpointSerializer(PathEndpointModelSerializerMixin):
+    pass
+
+
 #
 # Regions/sites
 #
@@ -207,7 +220,7 @@ class RegionSerializer(NautobotModelSerializer):
         ]
 
 
-class SiteSerializer(NautobotModelSerializer, TaggedObjectSerializer, StatusModelSerializerMixin):
+class SiteSerializer(NautobotModelSerializer, TaggedModelSerializerMixin, StatusModelSerializerMixin):
     url = serializers.HyperlinkedIdentityField(view_name="dcim-api:site-detail")
     region = NestedRegionSerializer(required=False, allow_null=True)
     tenant = NestedTenantSerializer(required=False, allow_null=True)
@@ -283,7 +296,7 @@ class LocationTypeSerializer(NautobotModelSerializer):
         ]
 
 
-class LocationSerializer(NautobotModelSerializer, TaggedObjectSerializer, StatusModelSerializerMixin):
+class LocationSerializer(NautobotModelSerializer, TaggedModelSerializerMixin, StatusModelSerializerMixin):
     url = serializers.HyperlinkedIdentityField(view_name="dcim-api:location-detail")
     location_type = NestedLocationTypeSerializer()
     parent = NestedLocationSerializer(required=False, allow_null=True)
@@ -384,7 +397,7 @@ class RackRoleSerializer(NautobotModelSerializer):
         ]
 
 
-class RackSerializer(NautobotModelSerializer, TaggedObjectSerializer, StatusModelSerializerMixin):
+class RackSerializer(NautobotModelSerializer, TaggedModelSerializerMixin, StatusModelSerializerMixin):
     url = serializers.HyperlinkedIdentityField(view_name="dcim-api:rack-detail")
     site = NestedSiteSerializer()
     location = NestedLocationSerializer(required=False, allow_null=True)
@@ -450,7 +463,7 @@ class RackUnitSerializer(serializers.Serializer):
     occupied = serializers.BooleanField(read_only=True)
 
 
-class RackReservationSerializer(NautobotModelSerializer, TaggedObjectSerializer):
+class RackReservationSerializer(NautobotModelSerializer, TaggedModelSerializerMixin):
     url = serializers.HyperlinkedIdentityField(view_name="dcim-api:rackreservation-detail")
     rack = NestedRackSerializer()
     user = NestedUserSerializer()
@@ -513,7 +526,7 @@ class ManufacturerSerializer(NautobotModelSerializer):
         ]
 
 
-class DeviceTypeSerializer(NautobotModelSerializer, TaggedObjectSerializer):
+class DeviceTypeSerializer(NautobotModelSerializer, TaggedModelSerializerMixin):
     url = serializers.HyperlinkedIdentityField(view_name="dcim-api:devicetype-detail")
     manufacturer = NestedManufacturerSerializer()
     subdevice_role = ChoiceField(choices=SubdeviceRoleChoices, allow_blank=True, required=False)
@@ -743,7 +756,7 @@ class PlatformSerializer(NautobotModelSerializer):
         ]
 
 
-class DeviceSerializer(NautobotModelSerializer, TaggedObjectSerializer, StatusModelSerializerMixin):
+class DeviceSerializer(NautobotModelSerializer, TaggedModelSerializerMixin, StatusModelSerializerMixin):
     url = serializers.HyperlinkedIdentityField(view_name="dcim-api:device-detail")
     device_type = NestedDeviceTypeSerializer()
     device_role = NestedDeviceRoleSerializer()
@@ -842,9 +855,9 @@ class DeviceNAPALMSerializer(serializers.Serializer):
 
 class ConsoleServerPortSerializer(
     NautobotModelSerializer,
-    TaggedObjectSerializer,
-    CableTerminationSerializer,
-    ConnectedEndpointSerializer,
+    TaggedModelSerializerMixin,
+    CableTerminationModelSerializerMixin,
+    PathEndpointModelSerializerMixin,
 ):
     url = serializers.HyperlinkedIdentityField(view_name="dcim-api:consoleserverport-detail")
     device = NestedDeviceSerializer()
@@ -871,9 +884,9 @@ class ConsoleServerPortSerializer(
 
 class ConsolePortSerializer(
     NautobotModelSerializer,
-    TaggedObjectSerializer,
-    CableTerminationSerializer,
-    ConnectedEndpointSerializer,
+    TaggedModelSerializerMixin,
+    CableTerminationModelSerializerMixin,
+    PathEndpointModelSerializerMixin,
 ):
     url = serializers.HyperlinkedIdentityField(view_name="dcim-api:consoleport-detail")
     device = NestedDeviceSerializer()
@@ -900,9 +913,9 @@ class ConsolePortSerializer(
 
 class PowerOutletSerializer(
     NautobotModelSerializer,
-    TaggedObjectSerializer,
-    CableTerminationSerializer,
-    ConnectedEndpointSerializer,
+    TaggedModelSerializerMixin,
+    CableTerminationModelSerializerMixin,
+    PathEndpointModelSerializerMixin,
 ):
     url = serializers.HyperlinkedIdentityField(view_name="dcim-api:poweroutlet-detail")
     device = NestedDeviceSerializer()
@@ -933,9 +946,9 @@ class PowerOutletSerializer(
 
 class PowerPortSerializer(
     NautobotModelSerializer,
-    TaggedObjectSerializer,
-    CableTerminationSerializer,
-    ConnectedEndpointSerializer,
+    TaggedModelSerializerMixin,
+    CableTerminationModelSerializerMixin,
+    PathEndpointModelSerializerMixin,
 ):
     url = serializers.HyperlinkedIdentityField(view_name="dcim-api:powerport-detail")
     device = NestedDeviceSerializer()
@@ -962,12 +975,31 @@ class PowerPortSerializer(
         ]
 
 
+class InterfaceCommonSerializer(NautobotModelSerializer, TaggedModelSerializerMixin):
+    def validate(self, data):
+
+        # Validate many-to-many VLAN assignments
+        mode = data.get("mode", getattr(self.instance, "mode", None))
+
+        if mode != InterfaceModeChoices.MODE_TAGGED:
+            if data.get("tagged_vlans"):
+                raise serializers.ValidationError(
+                    {
+                        "tagged_vlans": f"Mode must be set to {InterfaceModeChoices.MODE_TAGGED} when specifying tagged_vlans"
+                    }
+                )
+
+            if data.get("tagged_vlans") != [] and self.instance and self.instance.tagged_vlans.exists():
+                raise serializers.ValidationError({"tagged_vlans": f"Clear tagged_vlans to set mode to {mode}"})
+
+        return super().validate(data)
+
+
 # 2.0 TODO: This becomes non-default in 2.0, removed in 2.2.
 class InterfaceSerializerVersion12(
-    NautobotModelSerializer,
-    TaggedObjectSerializer,
-    CableTerminationSerializer,
-    ConnectedEndpointSerializer,
+    InterfaceCommonSerializer,
+    CableTerminationModelSerializerMixin,
+    PathEndpointModelSerializerMixin,
 ):
     url = serializers.HyperlinkedIdentityField(view_name="dcim-api:interface-detail")
     device = NestedDeviceSerializer()
@@ -1032,16 +1064,6 @@ class InterfaceSerializerVersion12(
                 )
 
         # Validate many-to-many VLAN assignments
-        mode = data.get("mode", getattr(self.instance, "mode", None))
-        has_tagged_vlans = data.get("tagged_vlans", False)
-        if not has_tagged_vlans and self.instance and self.instance.tagged_vlans.exists():
-            has_tagged_vlans = True
-
-        if has_tagged_vlans and mode != InterfaceModeChoices.MODE_TAGGED:
-            raise serializers.ValidationError(
-                {"tagged_vlans": f"Mode must be set to {InterfaceModeChoices.MODE_TAGGED} when specifying tagged_vlans"}
-            )
-
         device = self.instance.device if self.instance else data.get("device")
         for vlan in data.get("tagged_vlans", []):
             if vlan.site not in [device.site, None]:
@@ -1062,7 +1084,7 @@ class InterfaceSerializer(InterfaceSerializerVersion12, StatusModelSerializerMix
         fields.insert(4, "status")
 
 
-class RearPortSerializer(NautobotModelSerializer, TaggedObjectSerializer, CableTerminationSerializer):
+class RearPortSerializer(NautobotModelSerializer, TaggedModelSerializerMixin, CableTerminationModelSerializerMixin):
     url = serializers.HyperlinkedIdentityField(view_name="dcim-api:rearport-detail")
     device = NestedDeviceSerializer()
     type = ChoiceField(choices=PortTypeChoices)
@@ -1096,7 +1118,7 @@ class FrontPortRearPortSerializer(WritableNestedSerializer):
         fields = ["id", "url", "name", "label"]
 
 
-class FrontPortSerializer(NautobotModelSerializer, TaggedObjectSerializer, CableTerminationSerializer):
+class FrontPortSerializer(NautobotModelSerializer, TaggedModelSerializerMixin, CableTerminationModelSerializerMixin):
     url = serializers.HyperlinkedIdentityField(view_name="dcim-api:frontport-detail")
     device = NestedDeviceSerializer()
     type = ChoiceField(choices=PortTypeChoices)
@@ -1120,7 +1142,7 @@ class FrontPortSerializer(NautobotModelSerializer, TaggedObjectSerializer, Cable
         ]
 
 
-class DeviceBaySerializer(NautobotModelSerializer, TaggedObjectSerializer):
+class DeviceBaySerializer(NautobotModelSerializer, TaggedModelSerializerMixin):
     url = serializers.HyperlinkedIdentityField(view_name="dcim-api:devicebay-detail")
     device = NestedDeviceSerializer()
     installed_device = NestedDeviceSerializer(required=False, allow_null=True)
@@ -1137,7 +1159,7 @@ class DeviceBaySerializer(NautobotModelSerializer, TaggedObjectSerializer):
         ]
 
 
-class DeviceRedundancyGroupSerializer(NautobotModelSerializer, TaggedObjectSerializer, StatusModelSerializerMixin):
+class DeviceRedundancyGroupSerializer(NautobotModelSerializer, TaggedModelSerializerMixin, StatusModelSerializerMixin):
     url = serializers.HyperlinkedIdentityField(view_name="dcim-api:deviceredundancygroup-detail")
     failover_strategy = ChoiceField(choices=DeviceRedundancyGroupFailoverStrategyChoices)
 
@@ -1159,7 +1181,7 @@ class DeviceRedundancyGroupSerializer(NautobotModelSerializer, TaggedObjectSeria
 #
 
 
-class InventoryItemSerializer(NautobotModelSerializer, TaggedObjectSerializer):
+class InventoryItemSerializer(NautobotModelSerializer, TaggedModelSerializerMixin):
     url = serializers.HyperlinkedIdentityField(view_name="dcim-api:inventoryitem-detail")
     device = NestedDeviceSerializer()
     # Provide a default value to satisfy UniqueTogetherValidator
@@ -1190,7 +1212,7 @@ class InventoryItemSerializer(NautobotModelSerializer, TaggedObjectSerializer):
 #
 
 
-class CableSerializer(NautobotModelSerializer, TaggedObjectSerializer, StatusModelSerializerMixin):
+class CableSerializer(NautobotModelSerializer, TaggedModelSerializerMixin, StatusModelSerializerMixin):
     url = serializers.HyperlinkedIdentityField(view_name="dcim-api:cable-detail")
     termination_a_type = ContentTypeField(queryset=ContentType.objects.filter(CABLE_TERMINATION_MODELS))
     termination_b_type = ContentTypeField(queryset=ContentType.objects.filter(CABLE_TERMINATION_MODELS))
@@ -1342,7 +1364,7 @@ class InterfaceConnectionSerializer(ValidatedModelSerializer):
 #
 
 
-class VirtualChassisSerializer(NautobotModelSerializer, TaggedObjectSerializer):
+class VirtualChassisSerializer(NautobotModelSerializer, TaggedModelSerializerMixin):
     url = serializers.HyperlinkedIdentityField(view_name="dcim-api:virtualchassis-detail")
     master = NestedDeviceSerializer(required=False, allow_null=True)
     member_count = serializers.IntegerField(read_only=True)
@@ -1363,7 +1385,7 @@ class VirtualChassisSerializer(NautobotModelSerializer, TaggedObjectSerializer):
 #
 
 
-class PowerPanelSerializer(NautobotModelSerializer, TaggedObjectSerializer):
+class PowerPanelSerializer(NautobotModelSerializer, TaggedModelSerializerMixin):
     url = serializers.HyperlinkedIdentityField(view_name="dcim-api:powerpanel-detail")
     site = NestedSiteSerializer()
     location = NestedLocationSerializer(required=False, allow_null=True)
@@ -1384,9 +1406,9 @@ class PowerPanelSerializer(NautobotModelSerializer, TaggedObjectSerializer):
 
 class PowerFeedSerializer(
     NautobotModelSerializer,
-    TaggedObjectSerializer,
-    CableTerminationSerializer,
-    ConnectedEndpointSerializer,
+    TaggedModelSerializerMixin,
+    CableTerminationModelSerializerMixin,
+    PathEndpointModelSerializerMixin,
     StatusModelSerializerMixin,
 ):
     url = serializers.HyperlinkedIdentityField(view_name="dcim-api:powerfeed-detail")
