@@ -4,7 +4,6 @@ from django.conf import settings
 from django.urls import reverse
 from django.utils.http import urlencode
 
-from nautobot.utilities.utils import foreground_color
 from .choices import DeviceFaceChoices
 from .constants import RACK_ELEVATION_BORDER_WIDTH
 
@@ -46,13 +45,13 @@ class RackElevationSVG:
         )
 
     @staticmethod
-    def _add_gradient(drawing, id_, color):
+    def _add_gradient(drawing, id_, color, angle=45):
         gradient = drawing.linearGradient(
             start=(0, 0),
             end=(0, 25),
             spreadMethod="repeat",
             id_=id_,
-            gradientTransform="rotate(45, 0, 0)",
+            gradientTransform=f"rotate({angle}, 0, 0)",
             gradientUnits="userSpaceOnUse",
         )
         gradient.add_stop_color(offset="0%", color="#f7f7f7")
@@ -61,6 +60,16 @@ class RackElevationSVG:
         gradient.add_stop_color(offset="100%", color=color)
         drawing.defs.add(gradient)
 
+    @staticmethod
+    def _add_filters(drawing):
+        new_filter = drawing.filter(id="darkmodeinvert")
+        fct = new_filter.feComponentTransfer(color_interpolation_filters="sRGB", result="inverted")
+        fct.feFuncR("table", tableValues="1,0")
+        fct.feFuncG("table", tableValues="1,0")
+        fct.feFuncB("table", tableValues="1,0")
+        new_filter.feColorMatrix(type_="hueRotate", values="180", in_="inverted")
+        drawing.defs.add(new_filter)
+
     @staticmethod
     def _setup_drawing(width, height):
         drawing = svgwrite.Drawing(size=(width, height))
@@ -73,6 +82,8 @@ class RackElevationSVG:
         RackElevationSVG._add_gradient(drawing, "reserved", "#c7c7ff")
         RackElevationSVG._add_gradient(drawing, "occupied", "#d7d7d7")
         RackElevationSVG._add_gradient(drawing, "blocked", "#ffc0c0")
+        RackElevationSVG._add_gradient(drawing, "blocked_partial", "#a0a0a0", angle=-45)
+        RackElevationSVG._add_filters(drawing)
 
         return drawing
 
@@ -95,23 +106,6 @@ class RackElevationSVG:
         )
         link.set_desc(self._get_device_description(device))
         link.add(drawing.rect(start, end, style=f"fill: #{color}", class_="slot"))
-        hex_color = f"#{foreground_color(color)}"
-        link.add(
-            drawing.text(
-                device_fullname,
-                insert=text,
-                fill=hex_color,
-                class_=f"rack-device-fullname{'' if self.display_fullname else ' hidden'}",
-            )
-        )
-        link.add(
-            drawing.text(
-                device_shortname,
-                insert=text,
-                fill=hex_color,
-                class_=f"rack-device-shortname{' hidden' if self.display_fullname else ''}",
-            )
-        )
 
         # Embed front device type image if one exists
         if self.include_images and device.device_type.front_image:
@@ -124,6 +118,25 @@ class RackElevationSVG:
             image.fit(scale="slice")
             link.add(image)
 
+        link.add(
+            drawing.text(
+                device_fullname,
+                insert=text,
+                fill="white",
+                style="text-shadow: 1px 1px 3px black;",
+                class_=f"rack-device-fullname{'' if self.display_fullname else ' hidden'}",
+            )
+        )
+        link.add(
+            drawing.text(
+                device_shortname,
+                insert=text,
+                fill="white",
+                style="text-shadow: 1px 1px 3px black;",
+                class_=f"rack-device-shortname{' hidden' if self.display_fullname else ''}",
+            )
+        )
+
     def _draw_device_rear(self, drawing, device, start, end, text):
         rect = drawing.rect(start, end, class_="slot blocked")
         rect.set_desc(self._get_device_description(device))
@@ -132,18 +145,6 @@ class RackElevationSVG:
         device_shortname = settings.UI_RACK_VIEW_TRUNCATE_FUNCTION(str(device))
 
         drawing.add(rect)
-        drawing.add(
-            drawing.text(
-                device_fullname, insert=text, class_=f"rack-device-fullname{'' if self.display_fullname else ' hidden'}"
-            )
-        )
-        drawing.add(
-            drawing.text(
-                device_shortname,
-                insert=text,
-                class_=f"rack-device-shortname{' hidden' if self.display_fullname else ''}",
-            )
-        )
 
         # Embed rear device type image if one exists
         if self.include_images and device.device_type.rear_image:
@@ -156,6 +157,25 @@ class RackElevationSVG:
             image.fit(scale="slice")
             drawing.add(image)
 
+        drawing.add(
+            drawing.text(
+                device_fullname,
+                insert=text,
+                fill="white",
+                style="text-shadow: 1px 1px 3px black;",
+                class_=f"rack-device-fullname{'' if self.display_fullname else ' hidden'}",
+            )
+        )
+        drawing.add(
+            drawing.text(
+                device_shortname,
+                insert=text,
+                fill="white",
+                style="text-shadow: 1px 1px 3px black;",
+                class_=f"rack-device-shortname{' hidden' if self.display_fullname else ''}",
+            )
+        )
+
     @staticmethod
     def _draw_empty(drawing, rack, start, end, text, id_, face_id, class_, reservation):
         reverse_url = reverse("dcim:device_add")
@@ -189,7 +209,7 @@ class RackElevationSVG:
         unit_cursor = 0
         for u in elevation:
             o = other[unit_cursor]
-            if not u["device"] and o["device"] and o["device"].device_type.is_full_depth:
+            if not u["device"] and o["device"]:
                 u["device"] = o["device"]
                 u["height"] = 1
             unit_cursor += u.get("height", 1)
@@ -231,10 +251,24 @@ class RackElevationSVG:
             text_coordinates = (x_offset + (unit_width / 2), y_offset + end_y / 2)
 
             # Draw the device
-            if device and device.face == face and device.pk in self.permitted_device_ids:
-                self._draw_device_front(drawing, device, start_coordinates, end_coordinates, text_coordinates)
-            elif device and device.device_type.is_full_depth and device.pk in self.permitted_device_ids:
-                self._draw_device_rear(drawing, device, start_coordinates, end_coordinates, text_coordinates)
+            if device and device.pk in self.permitted_device_ids:
+                if device.face == face:
+                    self._draw_device_front(drawing, device, start_coordinates, end_coordinates, text_coordinates)
+                elif device.device_type.is_full_depth:
+                    self._draw_device_rear(drawing, device, start_coordinates, end_coordinates, text_coordinates)
+                else:
+                    # Half-depth devices add an empty box that is partially blocked
+                    self._draw_empty(
+                        drawing,
+                        self.rack,
+                        start_coordinates,
+                        end_coordinates,
+                        text_coordinates,
+                        unit["id"],
+                        face,
+                        "slot blocked_partial",
+                        None,
+                    )
             elif device:
                 # Devices which the user does not have permission to view are rendered only as unavailable space
                 drawing.add(drawing.rect(start_coordinates, end_coordinates, class_="blocked"))
