@@ -10,16 +10,16 @@ from nautobot.dcim.choices import (
     PowerOutletTypeChoices,
     PowerOutletFeedLegChoices,
     InterfaceTypeChoices,
+    InterfaceStatusChoices,
     PortTypeChoices,
 )
 
 from nautobot.core.models import BaseModel
 from nautobot.dcim.constants import REARPORT_POSITIONS_MAX, REARPORT_POSITIONS_MIN
-from nautobot.extras.models import CustomField, CustomFieldModel, ObjectChange, RelationshipModel
+from nautobot.extras.models import ChangeLoggedModel, CustomField, CustomFieldModel, RelationshipModel, Status
 from nautobot.extras.utils import extras_features
 from nautobot.utilities.fields import NaturalOrderingField
 from nautobot.utilities.ordering import naturalize_interface
-from nautobot.utilities.utils import serialize_object, serialize_object_v2
 from .device_components import (
     ConsolePort,
     ConsoleServerPort,
@@ -43,7 +43,7 @@ __all__ = (
 )
 
 
-class ComponentTemplateModel(BaseModel, CustomFieldModel, RelationshipModel):
+class ComponentTemplateModel(BaseModel, ChangeLoggedModel, CustomFieldModel, RelationshipModel):
     device_type = models.ForeignKey(to="dcim.DeviceType", on_delete=models.CASCADE, related_name="%(class)ss")
     name = models.CharField(max_length=64)
     _name = NaturalOrderingField(target_field="name", max_length=100, blank=True)
@@ -64,22 +64,17 @@ class ComponentTemplateModel(BaseModel, CustomFieldModel, RelationshipModel):
         """
         raise NotImplementedError()
 
-    def to_objectchange(self, action):
-        # Annotate the parent DeviceType
+    def to_objectchange(self, action, **kwargs):
+        """
+        Return a new ObjectChange with the `related_object` pinned to the `device_type` by default.
+        """
         try:
             device_type = self.device_type
         except ObjectDoesNotExist:
             # The parent DeviceType has already been deleted
             device_type = None
 
-        return ObjectChange(
-            changed_object=self,
-            object_repr=str(self),
-            action=action,
-            object_data=serialize_object(self),
-            object_data_v2=serialize_object_v2(self),
-            related_object=device_type,
-        )
+        return super().to_objectchange(action, related_object=device_type, **kwargs)
 
     def instantiate_model(self, model, device, **kwargs):
         """
@@ -264,11 +259,16 @@ class InterfaceTemplate(ComponentTemplateModel):
         unique_together = ("device_type", "name")
 
     def instantiate(self, device):
+        try:
+            status = Status.objects.get_for_model(Interface).get(slug=InterfaceStatusChoices.STATUS_ACTIVE)
+        except Status.DoesNotExist:
+            status = Status.objects.get_for_model(Interface).first()
         return self.instantiate_model(
             model=Interface,
             device=device,
             type=self.type,
             mgmt_only=self.mgmt_only,
+            status=status,
         )
 
 
