@@ -9,6 +9,7 @@ from nautobot.dcim.choices import (
     CableStatusChoices,
     CableTypeChoices,
     DeviceFaceChoices,
+    InterfaceModeChoices,
     InterfaceTypeChoices,
     PortTypeChoices,
     PowerOutletFeedLegChoices,
@@ -1320,6 +1321,9 @@ class InterfaceTestCase(TestCase):
             site=site,
             status=status,
         )
+        self.other_site_vlan = VLAN.objects.create(
+            name="Other Site VLAN", vid=100, site=Site.objects.create(name="Other Site")
+        )
 
     def test_tagged_vlan_raise_error_if_mode_not_set_to_tagged(self):
         interface = Interface.objects.create(
@@ -1333,6 +1337,20 @@ class InterfaceTestCase(TestCase):
             err.exception.message_dict["tagged_vlans"][0], "Mode must be set to tagged when specifying tagged_vlans"
         )
 
+    def test_error_raised_when_adding_tagged_vlan_with_different_site_from_interface_parent_site(self):
+        with self.assertRaises(ValidationError) as err:
+            interface = Interface.objects.create(
+                name="Test Interface",
+                mode=InterfaceModeChoices.MODE_TAGGED,
+                device=self.device,
+            )
+            interface.tagged_vlans.add(self.other_site_vlan)
+        self.assertEqual(
+            err.exception.message_dict["tagged_vlans"][0],
+            f"Tagged VLAN with names {[self.other_site_vlan.name]} must all belong to the "
+            f"same site as the interface's parent device, or it must be global.",
+        )
+
 
 class SiteTestCase(TestCase):
     def test_latitude_or_longitude(self):
