@@ -4,12 +4,12 @@ Class-modifying mixins that need to be standalone to avoid circular imports.
 import sys
 
 from cacheops.utils import family_has_profile
-from django.conf import settings
 from django.db.models import Q
 from django.urls import NoReverseMatch, reverse
 from funcy import once_per
 
 from nautobot.utilities.forms.fields import DynamicModelMultipleChoiceField
+from nautobot.utilities.utils import get_route_for_model
 
 
 class DynamicGroupMixin:
@@ -28,6 +28,23 @@ class DynamicGroupMixin:
 
         return self._dynamic_group_queryset
 
+    def get_dynamic_groups_url(self):
+        """Return the dynamic groups URL for a given instance."""
+        route = get_route_for_model(self, "dynamicgroups")
+
+        # Iterate the pk-like fields and try to get a URL, or return None.
+        fields = ["pk", "slug"]
+        for field in fields:
+            if not hasattr(self, field):
+                continue
+
+            try:
+                return reverse(route, kwargs={field: getattr(self, field)})
+            except NoReverseMatch:
+                continue
+
+        return None
+
 
 class NotesMixin:
     """
@@ -46,12 +63,8 @@ class NotesMixin:
         return self._notes_queryset
 
     def get_notes_url(self):
-        """Return the changelog URL for a given instance."""
-        meta = self._meta
-
-        route = f"{meta.app_label}:{meta.model_name}_notes"
-        if meta.app_label in settings.PLUGINS:
-            route = f"plugins:{route}"
+        """Return the notes URL for a given instance."""
+        route = get_route_for_model(self, "notes")
 
         # Iterate the pk-like fields and try to get a URL, or return None.
         fields = ["pk", "slug"]
