@@ -4,6 +4,7 @@ from django.conf import settings
 from django.contrib.contenttypes.models import ContentType
 from django.core.exceptions import ValidationError
 from django.db.models import ProtectedError
+from django.test import override_settings
 from django.urls import reverse
 from rest_framework import status
 
@@ -13,13 +14,16 @@ from nautobot.dcim.models import Site, Rack, Device
 from nautobot.dcim.tables import SiteTable
 from nautobot.extras.choices import CustomFieldTypeChoices, CustomFieldFilterLogicChoices
 from nautobot.extras.models import ComputedField, CustomField, CustomFieldChoice, Status
+from nautobot.users.models import ObjectPermission
 from nautobot.utilities.tables import CustomFieldColumn
 from nautobot.utilities.testing import APITestCase, CeleryTestCase, TestCase
+from nautobot.utilities.testing.utils import post_data
 from nautobot.virtualization.models import VirtualMachine
 
 
 class CustomFieldTest(TestCase):
     def setUp(self):
+        super().setUp()
         active_status = Status.objects.get_for_model(Site).get(slug="active")
         Site.objects.create(name="Site A", slug="site-a", status=active_status)
         Site.objects.create(name="Site B", slug="site-b", status=active_status)
@@ -125,7 +129,7 @@ class CustomFieldTest(TestCase):
             self.assertEqual(cf.slug, cf.name)
 
             # Assign a value to the first Site
-            site = Site.objects.first()
+            site = Site.objects.get(slug="site-a")
             # 2.0 TODO: #824 cf.slug rather than cf.name
             site.cf[cf.name] = data["field_value"]
             site.validated_save()
@@ -163,7 +167,7 @@ class CustomFieldTest(TestCase):
         CustomFieldChoice.objects.create(field=cf, value="Option C")
 
         # Assign a value to the first Site
-        site = Site.objects.first()
+        site = Site.objects.get(slug="site-a")
         # 2.0 TODO: #824 cf.slug rather than cf.name
         site.cf[cf.name] = "Option A"
         site.validated_save()
@@ -201,7 +205,7 @@ class CustomFieldTest(TestCase):
         CustomFieldChoice.objects.create(field=cf, value="Option C")
 
         # Assign a value to the first Site
-        site = Site.objects.first()
+        site = Site.objects.get(slug="site-a")
         # 2.0 TODO: #824 cf.slug rather than cf.name
         site.cf[cf.name] = ["Option A", "Option B"]
         site.validated_save()
@@ -222,6 +226,65 @@ class CustomFieldTest(TestCase):
         # Delete the custom field
         cf.delete()
 
+    def test_multi_select_field_value_after_bulk_update(self):
+        obj_type = ContentType.objects.get_for_model(Site)
+
+        # Create a custom field
+        cf = CustomField(
+            type=CustomFieldTypeChoices.TYPE_MULTISELECT,
+            name="my_field",
+            required=False,
+        )
+        cf.save()
+        cf.content_types.set([obj_type])
+        CustomFieldChoice.objects.create(field=cf, value="Option A")
+        CustomFieldChoice.objects.create(field=cf, value="Option B")
+        CustomFieldChoice.objects.create(field=cf, value="Option C")
+        cf.validated_save()
+
+        # Assign values to all sites
+        sites = Site.objects.all()
+        # 2.0 TODO: #824 cf.slug rather than cf.name
+        for site in sites:
+            site.cf[cf.name] = ["Option A", "Option B", "Option C"]
+            site.validated_save()
+
+            # Retrieve the stored value
+            site.refresh_from_db()
+            # 2.0 TODO: #824 cf.slug rather than cf.name
+            self.assertEqual(site.cf[cf.name], ["Option A", "Option B", "Option C"])
+
+        pk_list = list(Site.objects.values_list("pk", flat=True))
+        data = {
+            "pk": pk_list,
+            "_apply": True,  # Form button
+        }
+        # set my_field to [] to emulate form submission when the user does not make any changes to the multiselect cf.
+        bulk_edit_data = {
+            f"cf_{cf.slug}": [],
+        }
+        # Append the form data to the request
+        data.update(post_data(bulk_edit_data))
+        # Assign model-level permission
+        obj_perm = ObjectPermission(
+            name="Test permission",
+            actions=["view", "change"],
+        )
+        obj_perm.save()
+        obj_perm.users.add(self.user)
+        obj_perm.object_types.add(ContentType.objects.get_for_model(Site))
+
+        # Try POST with model-level permission
+        bulk_edit_url = reverse("dcim:site_bulk_edit")
+        self.assertHttpStatus(self.client.post(bulk_edit_url, data), 302)
+
+        # Assert the values are unchanged after bulk edit
+        for site in sites:
+            site.refresh_from_db()
+            self.assertEqual(site.cf[cf.name], ["Option A", "Option B", "Option C"])
+
+        cf.delete()
+
     def test_text_field_value(self):
         obj_type = ContentType.objects.get_for_model(Site)
 
@@ -235,7 +298,7 @@ class CustomFieldTest(TestCase):
         cf.content_types.set([obj_type])
 
         # Assign a disallowed value (list) to the first Site
-        site = Site.objects.first()
+        site = Site.objects.get(slug="site-a")
         # 2.0 TODO: #824 cf.slug rather than cf.name
         site.cf[cf.name] = ["I", "am", "a", "list"]
         with self.assertRaises(ValidationError) as context:
@@ -267,6 +330,52 @@ class CustomFieldTest(TestCase):
         # Delete the custom field
         cf.delete()
 
+    @override_settings(
+        CELERY_TASK_ALWAYS_EAGER=True,
+        CELERY_TASK_EAGER_PROPOGATES=True,
+        CELERY_BROKER_URL="memory://",
+        CELERY_BACKEND="memory",
+    )
+    def test_regex_validation(self):
+        obj_type = ContentType.objects.get_for_model(Site)
+
+        for cf_type in CustomFieldTypeChoices.REGEX_TYPES:
+            # validation for select and multi-select are performed on the CustomFieldChoice model
+            if "select" in cf_type:
+                continue
+
+            # Create a custom field
+            cf = CustomField(
+                type=cf_type,
+                name=f"cf_test_{cf_type}",
+                required=False,
+                validation_regex="A.C[01]x?",
+            )
+            cf.save()
+            cf.content_types.set([obj_type])
+
+            # Assign values to the first Site
+            site = Site.objects.first()
+
+            non_matching_values = ["abc1", "AC1", "00AbC", "abc1x", "00abc1x00"]
+            error_message = f"Value must match regex '{cf.validation_regex}'"
+            for value in non_matching_values:
+                with self.subTest(cf_type=cf_type, value=value):
+                    with self.assertRaisesMessage(ValidationError, error_message):
+                        # 2.0 TODO: #824 cf.slug rather than cf.name
+                        site.cf[cf.name] = value
+                        site.validated_save()
+
+            matching_values = ["ABC1", "00AbC0", "00ABC0x00"]
+            for value in matching_values:
+                with self.subTest(cf_type=cf_type, value=value):
+                    # 2.0 TODO: #824 cf.slug rather than cf.name
+                    site.cf[cf.name] = value
+                    site.validated_save()
+
+            # Delete the custom field
+            cf.delete()
+
 
 class CustomFieldManagerTest(TestCase):
     def setUp(self):
@@ -1388,105 +1497,331 @@ class CustomFieldFilterTest(TestCase):
         Site.objects.create(name="Site 4", slug="site-4", _custom_field_data={})
 
     def test_filter_integer(self):
-        self.assertEqual(self.filterset({"cf_cf1": 100}, self.queryset).qs.count(), 1)
-        self.assertEqual(self.filterset({"cf_cf1__n": [100]}, self.queryset).qs.count(), 3)
-        self.assertEqual(self.filterset({"cf_cf1__lte": [101]}, self.queryset).qs.count(), 1)
-        self.assertEqual(self.filterset({"cf_cf1__lt": [101]}, self.queryset).qs.count(), 1)
-        self.assertEqual(self.filterset({"cf_cf1__gte": [199]}, self.queryset).qs.count(), 1)
-        self.assertEqual(self.filterset({"cf_cf1__gt": [199]}, self.queryset).qs.count(), 1)
+        self.assertQuerysetEqual(
+            self.filterset({"cf_cf1": 100}, self.queryset).qs,
+            self.queryset.filter(_custom_field_data__cf1=100),
+        )
+        self.assertQuerysetEqual(
+            self.filterset({"cf_cf1__n": [100]}, self.queryset).qs,
+            self.queryset.exclude(_custom_field_data__cf1=100)
+            | self.queryset.filter(_custom_field_data__cf1__isnull=True),
+        )
+        self.assertQuerysetEqual(
+            self.filterset({"cf_cf1__lte": [101]}, self.queryset).qs,
+            self.queryset.filter(_custom_field_data__cf1__lte=100),
+        )
+        self.assertQuerysetEqual(
+            self.filterset({"cf_cf1__lt": [101]}, self.queryset).qs,
+            self.queryset.filter(_custom_field_data__cf1__lt=101),
+        )
+        self.assertQuerysetEqual(
+            self.filterset({"cf_cf1__gte": [199]}, self.queryset).qs,
+            self.queryset.filter(_custom_field_data__cf1__gte=199),
+        )
+        self.assertQuerysetEqual(
+            self.filterset({"cf_cf1__gt": [199]}, self.queryset).qs,
+            self.queryset.filter(_custom_field_data__cf1__gt=199),
+        )
 
     def test_filter_boolean(self):
-        self.assertEqual(self.filterset({"cf_cf2": True}, self.queryset).qs.count(), 1)
-        self.assertEqual(self.filterset({"cf_cf2": False}, self.queryset).qs.count(), 1)
+        self.assertQuerysetEqual(
+            self.filterset({"cf_cf2": True}, self.queryset).qs, self.queryset.filter(_custom_field_data__cf2=True)
+        )
+        self.assertQuerysetEqual(
+            self.filterset({"cf_cf2": False}, self.queryset).qs, self.queryset.filter(_custom_field_data__cf2=False)
+        )
 
     def test_filter_text(self):
-        self.assertEqual(self.filterset({"cf_cf3": "foo"}, self.queryset).qs.count(), 1)
-        self.assertEqual(self.filterset({"cf_cf4": "foo"}, self.queryset).qs.count(), 2)
-        self.assertEqual(self.filterset({"cf_cf4__n": ["foo"]}, self.queryset).qs.count(), 3)
-        self.assertEqual(self.filterset({"cf_cf4__ic": ["OOB"]}, self.queryset).qs.count(), 1)
-        self.assertEqual(self.filterset({"cf_cf4__nic": ["OOB"]}, self.queryset).qs.count(), 3)
-        self.assertEqual(self.filterset({"cf_cf4__iew": ["Bar"]}, self.queryset).qs.count(), 1)
-        self.assertEqual(self.filterset({"cf_cf4__niew": ["Bar"]}, self.queryset).qs.count(), 3)
-        self.assertEqual(self.filterset({"cf_cf4__isw": ["Foob"]}, self.queryset).qs.count(), 1)
-        self.assertEqual(self.filterset({"cf_cf4__nisw": ["Foob"]}, self.queryset).qs.count(), 3)
-        self.assertEqual(self.filterset({"cf_cf4__ie": ["Foo"]}, self.queryset).qs.count(), 1)
-        self.assertEqual(self.filterset({"cf_cf4__nie": ["Foo"]}, self.queryset).qs.count(), 3)
-        self.assertEqual(self.filterset({"cf_cf4__re": ["f.*b"]}, self.queryset).qs.count(), 1)
-        self.assertEqual(self.filterset({"cf_cf4__nre": ["f.*b"]}, self.queryset).qs.count(), 3)
-        self.assertEqual(self.filterset({"cf_cf4__ire": ["F.*b"]}, self.queryset).qs.count(), 1)
-        self.assertEqual(self.filterset({"cf_cf4__nire": ["F.*b"]}, self.queryset).qs.count(), 3)
+        self.assertQuerysetEqual(
+            self.filterset({"cf_cf3": "foo"}, self.queryset).qs,
+            self.queryset.filter(_custom_field_data__cf3__contains="foo"),
+        )
+        self.assertQuerysetEqual(
+            self.filterset({"cf_cf4": "foo"}, self.queryset).qs,
+            self.queryset.filter(_custom_field_data__cf4__icontains="foo"),
+        )
+        self.assertQuerysetEqual(
+            self.filterset({"cf_cf4__n": ["foo"]}, self.queryset).qs,
+            self.queryset.exclude(_custom_field_data__cf4="foo")
+            | self.queryset.filter(_custom_field_data__cf4__isnull=True),
+        )
+        self.assertQuerysetEqual(
+            self.filterset({"cf_cf4__ic": ["OOB"]}, self.queryset).qs,
+            self.queryset.filter(_custom_field_data__cf4__icontains="OOB"),
+        )
+        self.assertQuerysetEqual(
+            self.filterset({"cf_cf4__nic": ["OOB"]}, self.queryset).qs,
+            self.queryset.exclude(_custom_field_data__cf4__icontains="OOB")
+            | self.queryset.filter(_custom_field_data__cf4__isnull=True),
+        )
+        self.assertQuerysetEqual(
+            self.filterset({"cf_cf4__iew": ["Bar"]}, self.queryset).qs,
+            self.queryset.filter(_custom_field_data__cf4__iendswith="Bar"),
+        )
+        self.assertQuerysetEqual(
+            self.filterset({"cf_cf4__niew": ["Bar"]}, self.queryset).qs,
+            self.queryset.exclude(_custom_field_data__cf4__iendswith="Bar")
+            | self.queryset.filter(_custom_field_data__cf4__isnull=True),
+        )
+        self.assertQuerysetEqual(
+            self.filterset({"cf_cf4__isw": ["Foob"]}, self.queryset).qs,
+            self.queryset.filter(_custom_field_data__cf4__istartswith="Foob"),
+        )
+        self.assertQuerysetEqual(
+            self.filterset({"cf_cf4__nisw": ["Foob"]}, self.queryset).qs,
+            self.queryset.exclude(_custom_field_data__cf4__istartswith="Foob")
+            | self.queryset.filter(_custom_field_data__cf4__isnull=True),
+        )
+        self.assertQuerysetEqual(
+            self.filterset({"cf_cf4__ie": ["Foo"]}, self.queryset).qs,
+            self.queryset.filter(_custom_field_data__cf4__iexact="Foo"),
+        )
+        self.assertQuerysetEqual(
+            self.filterset({"cf_cf4__nie": ["Foo"]}, self.queryset).qs,
+            self.queryset.exclude(_custom_field_data__cf4__iexact="Foo")
+            | self.queryset.filter(_custom_field_data__cf4__isnull=True),
+        )
+        self.assertQuerysetEqual(
+            self.filterset({"cf_cf4__re": ["f.*b"]}, self.queryset).qs,
+            self.queryset.filter(_custom_field_data__cf4__regex="f.*b"),
+        )
+        self.assertQuerysetEqual(
+            self.filterset({"cf_cf4__nre": ["f.*b"]}, self.queryset).qs,
+            self.queryset.exclude(_custom_field_data__cf4__regex="f.*b")
+            | self.queryset.filter(_custom_field_data__cf4__isnull=True),
+        )
+        self.assertQuerysetEqual(
+            self.filterset({"cf_cf4__ire": ["F.*b"]}, self.queryset).qs,
+            self.queryset.filter(_custom_field_data__cf4__iregex="F.*b"),
+        )
+        self.assertQuerysetEqual(
+            self.filterset({"cf_cf4__nire": ["F.*b"]}, self.queryset).qs,
+            self.queryset.exclude(_custom_field_data__cf4__iregex="F.*b")
+            | self.queryset.filter(_custom_field_data__cf4__isnull=True),
+        )
 
     def test_filter_date(self):
-        self.assertEqual(self.filterset({"cf_cf5": "2016-06-26"}, self.queryset).qs.count(), 1)
-        self.assertEqual(self.filterset({"cf_cf5__n": ["2016-06-26"]}, self.queryset).qs.count(), 3)
-        self.assertEqual(self.filterset({"cf_cf5__lte": ["2016-06-28"]}, self.queryset).qs.count(), 2)
-        self.assertEqual(self.filterset({"cf_cf5__lte": ["2016-06-27"]}, self.queryset).qs.count(), 2)
-        self.assertEqual(self.filterset({"cf_cf5__lte": ["2016-06-26"]}, self.queryset).qs.count(), 1)
-        self.assertEqual(self.filterset({"cf_cf5__lte": ["2016-06-25"]}, self.queryset).qs.count(), 0)
-        self.assertEqual(self.filterset({"cf_cf5__gte": ["2016-06-25"]}, self.queryset).qs.count(), 2)
-        self.assertEqual(self.filterset({"cf_cf5__gte": ["2016-06-26"]}, self.queryset).qs.count(), 2)
-        self.assertEqual(self.filterset({"cf_cf5__gte": ["2016-06-27"]}, self.queryset).qs.count(), 1)
-        self.assertEqual(self.filterset({"cf_cf5__gte": ["2016-06-28"]}, self.queryset).qs.count(), 0)
+        self.assertQuerysetEqual(
+            self.filterset({"cf_cf5": "2016-06-26"}, self.queryset).qs,
+            self.queryset.filter(_custom_field_data__cf5="2016-06-26"),
+        )
+        self.assertQuerysetEqual(
+            self.filterset({"cf_cf5__n": "2016-06-26"}, self.queryset).qs,
+            self.queryset.exclude(_custom_field_data__cf5="2016-06-26")
+            | self.queryset.filter(_custom_field_data__cf4__isnull=True),
+        )
+        self.assertQuerysetEqual(
+            self.filterset({"cf_cf5__lte": ["2016-06-28"]}, self.queryset).qs,
+            self.queryset.filter(_custom_field_data__cf5__lte="2016-06-28"),
+        )
+        self.assertQuerysetEqual(
+            self.filterset({"cf_cf5__lte": ["2016-06-27"]}, self.queryset).qs,
+            self.queryset.filter(_custom_field_data__cf5__lte="2016-06-27"),
+        )
+        self.assertQuerysetEqual(
+            self.filterset({"cf_cf5__lte": ["2016-06-26"]}, self.queryset).qs,
+            self.queryset.filter(_custom_field_data__cf5__lte="2016-06-26"),
+        )
+        self.assertQuerysetEqual(
+            self.filterset({"cf_cf5__lte": ["2016-06-25"]}, self.queryset).qs,
+            self.queryset.filter(_custom_field_data__lte="2016-06-25"),
+        )
+        self.assertQuerysetEqual(
+            self.filterset({"cf_cf5__gte": ["2016-06-25"]}, self.queryset).qs,
+            self.queryset.filter(_custom_field_data__cf5__gte="2016-06-25"),
+        )
+        self.assertQuerysetEqual(
+            self.filterset({"cf_cf5__gte": ["2016-06-26"]}, self.queryset).qs,
+            self.queryset.filter(_custom_field_data__cf5__gte="2016-06-26"),
+        )
+        self.assertQuerysetEqual(
+            self.filterset({"cf_cf5__gte": ["2016-06-27"]}, self.queryset).qs,
+            self.queryset.filter(_custom_field_data__cf5__gte="2016-06-27"),
+        )
+        self.assertQuerysetEqual(
+            self.filterset({"cf_cf5__gte": ["2016-06-28"]}, self.queryset).qs,
+            self.queryset.filter(_custom_field_data__cf5__gte="2016-06-28"),
+        )
         params = {"cf_cf5__gte": ["2016-06-25"], "cf_cf5__lt": ["2016-06-27"]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)
+        self.assertQuerysetEqual(
+            self.filterset(params, self.queryset).qs,
+            self.queryset.filter(_custom_field_data__cf5__gte="2016-06-25", _custom_field_data__cf5__lt="2016-06-27"),
+        )
 
     def test_filter_url(self):
         params = {"cf_cf6": "http://foo.example.com/"}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)
+        self.assertQuerysetEqual(
+            self.filterset(params, self.queryset).qs,
+            self.queryset.filter(_custom_field_data__cf6="http://foo.example.com/"),
+        )
         params = {"cf_cf6__n": ["http://foo.example.com/"]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 3)
+        self.assertQuerysetEqual(
+            self.filterset(params, self.queryset).qs,
+            self.queryset.exclude(_custom_field_data__cf6="http://foo.example.com/")
+            | self.queryset.filter(_custom_field_data__cf6__isnull=True),
+        )
         params = {"cf_cf7": "example.com"}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
+        self.assertQuerysetEqual(
+            self.filterset(params, self.queryset).qs,
+            self.queryset.filter(_custom_field_data__cf7__icontains="example.com"),
+        )
         params = {"cf_cf7__n": ["http://foo.example.com/"]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 3)
+        self.assertQuerysetEqual(
+            self.filterset(params, self.queryset).qs,
+            self.queryset.exclude(_custom_field_data__cf7="http://foo.example.com/")
+            | self.queryset.filter(_custom_field_data__cf7__isnull=True),
+        )
         params = {"cf_cf6__ic": ["FOO.example.COM"]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)
+        self.assertQuerysetEqual(
+            self.filterset(params, self.queryset).qs,
+            self.queryset.filter(_custom_field_data__cf6__icontains="FOO.example.COM"),
+        )
         params = {"cf_cf6__nic": ["FOO.example.COM"]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 3)
+        self.assertQuerysetEqual(
+            self.filterset(params, self.queryset).qs,
+            self.queryset.exclude(_custom_field_data__cf6__icontains="FOO.example.COM")
+            | self.queryset.filter(_custom_field_data__cf6__isnull=True),
+        )
         params = {"cf_cf6__iew": ["FOO.example.COM/"]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)
+        self.assertQuerysetEqual(
+            self.filterset(params, self.queryset).qs,
+            self.queryset.filter(_custom_field_data__cf6__iendswith="FOO.example.COM/"),
+        )
         params = {"cf_cf6__niew": ["FOO.example.COM/"]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 3)
+        self.assertQuerysetEqual(
+            self.filterset(params, self.queryset).qs,
+            self.queryset.exclude(_custom_field_data__cf6__iendswith="FOO.example.COM/")
+            | self.queryset.filter(_custom_field_data__cf6__isnull=True),
+        )
         params = {"cf_cf6__isw": ["HTTP://FOO"]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)
+        self.assertQuerysetEqual(
+            self.filterset(params, self.queryset).qs,
+            self.queryset.filter(_custom_field_data__cf6__istartswith="HTTP://FOO"),
+        )
         params = {"cf_cf6__nisw": ["HTTP://FOO"]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 3)
+        self.assertQuerysetEqual(
+            self.filterset(params, self.queryset).qs,
+            self.queryset.exclude(_custom_field_data__cf6__istartswith="HTTP://FOO")
+            | self.queryset.filter(_custom_field_data__cf6__isnull=True),
+        )
         params = {"cf_cf6__ie": ["http://FOO.example.COM/"]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)
+        self.assertQuerysetEqual(
+            self.filterset(params, self.queryset).qs,
+            self.queryset.filter(_custom_field_data__cf6__iexact="http://FOO.example.COM/"),
+        )
         params = {"cf_cf6__nie": ["http://FOO.example.COM/"]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 3)
+        self.assertQuerysetEqual(
+            self.filterset(params, self.queryset).qs,
+            self.queryset.exclude(_custom_field_data__cf6__iexact="http://FOO.example.COM/")
+            | self.queryset.filter(_custom_field_data__cf6__isnull=True),
+        )
         params = {"cf_cf6__re": ["foo.*com"]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)
+        self.assertQuerysetEqual(
+            self.filterset(params, self.queryset).qs,
+            self.queryset.filter(_custom_field_data__cf6__regex="foo.*com"),
+        )
         params = {"cf_cf6__nre": ["foo.*com"]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 3)
+        self.assertQuerysetEqual(
+            self.filterset(params, self.queryset).qs,
+            self.queryset.exclude(_custom_field_data__cf6__regex="foo.*com")
+            | self.queryset.filter(_custom_field_data__cf6__isnull=True),
+        )
         params = {"cf_cf6__ire": ["FOO.*COM"]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)
+        self.assertQuerysetEqual(
+            self.filterset(params, self.queryset).qs,
+            self.queryset.filter(_custom_field_data__cf6__iregex="FOO.*COM"),
+        )
         params = {"cf_cf6__nire": ["FOO.*COM"]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 3)
+        self.assertQuerysetEqual(
+            self.filterset(params, self.queryset).qs,
+            self.queryset.exclude(_custom_field_data__cf6__iregex="FOO.*COM")
+            | self.queryset.filter(_custom_field_data__cf6__isnull=True),
+        )
 
     def test_filter_select(self):
-        self.assertEqual(self.filterset({"cf_cf8": "Foo"}, self.queryset).qs.count(), 1)
-        self.assertEqual(self.filterset({"cf_cf8__n": ["Foo"]}, self.queryset).qs.count(), 3)
-        self.assertEqual(self.filterset({"cf_cf8__ic": ["FOO"]}, self.queryset).qs.count(), 1)
-        self.assertEqual(self.filterset({"cf_cf8__nic": ["FOO"]}, self.queryset).qs.count(), 3)
-        self.assertEqual(self.filterset({"cf_cf8__iew": ["AR"]}, self.queryset).qs.count(), 1)
-        self.assertEqual(self.filterset({"cf_cf8__niew": ["AR"]}, self.queryset).qs.count(), 3)
-        self.assertEqual(self.filterset({"cf_cf8__isw": ["FO"]}, self.queryset).qs.count(), 1)
-        self.assertEqual(self.filterset({"cf_cf8__nisw": ["FO"]}, self.queryset).qs.count(), 3)
-        self.assertEqual(self.filterset({"cf_cf8__ie": ["foo"]}, self.queryset).qs.count(), 1)
-        self.assertEqual(self.filterset({"cf_cf8__nie": ["foo"]}, self.queryset).qs.count(), 3)
-        self.assertEqual(self.filterset({"cf_cf8__re": ["F.o"]}, self.queryset).qs.count(), 1)
-        self.assertEqual(self.filterset({"cf_cf8__nre": ["F.o"]}, self.queryset).qs.count(), 3)
-        self.assertEqual(self.filterset({"cf_cf8__ire": ["F.O"]}, self.queryset).qs.count(), 1)
-        self.assertEqual(self.filterset({"cf_cf8__nire": ["F.O"]}, self.queryset).qs.count(), 3)
+        self.assertQuerysetEqual(
+            self.filterset({"cf_cf8": "Foo"}, self.queryset).qs,
+            self.queryset.filter(_custom_field_data__cf8="Foo"),
+        )
+        self.assertQuerysetEqual(
+            self.filterset({"cf_cf8__n": ["Foo"]}, self.queryset).qs,
+            self.queryset.exclude(_custom_field_data__cf8="Foo")
+            | self.queryset.filter(_custom_field_data__cf8__isnull=True),
+        )
+        self.assertQuerysetEqual(
+            self.filterset({"cf_cf8__ic": ["FOO"]}, self.queryset).qs,
+            self.queryset.filter(_custom_field_data__cf8__icontains="FOO"),
+        )
+        self.assertQuerysetEqual(
+            self.filterset({"cf_cf8__nic": ["FOO"]}, self.queryset).qs,
+            self.queryset.exclude(_custom_field_data__cf8__icontains="FOO")
+            | self.queryset.filter(_custom_field_data__cf8__isnull=True),
+        )
+        self.assertQuerysetEqual(
+            self.filterset({"cf_cf8__iew": ["AR"]}, self.queryset).qs,
+            self.queryset.filter(_custom_field_data__cf8__iendswith="AR"),
+        )
+        self.assertQuerysetEqual(
+            self.filterset({"cf_cf8__niew": ["AR"]}, self.queryset).qs,
+            self.queryset.exclude(_custom_field_data__cf8__iendswith="AR")
+            | self.queryset.filter(_custom_field_data__cf8__isnull=True),
+        )
+        self.assertQuerysetEqual(
+            self.filterset({"cf_cf8__isw": ["FO"]}, self.queryset).qs,
+            self.queryset.filter(_custom_field_data__cf8__istartswith="FO"),
+        )
+        self.assertQuerysetEqual(
+            self.filterset({"cf_cf8__nisw": ["FO"]}, self.queryset).qs,
+            self.queryset.exclude(_custom_field_data__cf8__istartswith="FO")
+            | self.queryset.filter(_custom_field_data__cf8__isnull=True),
+        )
+        self.assertQuerysetEqual(
+            self.filterset({"cf_cf8__ie": ["foo"]}, self.queryset).qs,
+            self.queryset.filter(_custom_field_data__cf8__iexact="foo"),
+        )
+        self.assertQuerysetEqual(
+            self.filterset({"cf_cf8__nie": ["foo"]}, self.queryset).qs,
+            self.queryset.exclude(_custom_field_data__cf8__istartswith="FO")
+            | self.queryset.filter(_custom_field_data__cf8__isnull=True),
+        )
+        self.assertQuerysetEqual(
+            self.filterset({"cf_cf8__re": ["F.o"]}, self.queryset).qs,
+            self.queryset.filter(_custom_field_data__cf8__regex="F.o"),
+        )
+        self.assertQuerysetEqual(
+            self.filterset({"cf_cf8__nre": ["F.o"]}, self.queryset).qs,
+            self.queryset.exclude(_custom_field_data__cf8__regex="F.o")
+            | self.queryset.filter(_custom_field_data__cf8__isnull=True),
+        )
+        self.assertQuerysetEqual(
+            self.filterset({"cf_cf8__ire": ["F.O"]}, self.queryset).qs,
+            self.queryset.filter(_custom_field_data__cf8__iregex="F.o"),
+        )
+        self.assertQuerysetEqual(
+            self.filterset({"cf_cf8__nire": ["F.O"]}, self.queryset).qs,
+            self.queryset.exclude(_custom_field_data__cf8__iregex="F.o")
+            | self.queryset.filter(_custom_field_data__cf8__isnull=True),
+        )
 
     def test_filter_multi_select(self):
-        self.assertEqual(self.filterset({"cf_cf9": "Foo"}, self.queryset).qs.count(), 2)
-        self.assertEqual(self.filterset({"cf_cf9": "Bar"}, self.queryset).qs.count(), 1)
+        self.assertQuerysetEqual(
+            self.filterset({"cf_cf9": "Foo"}, self.queryset).qs,
+            self.queryset.filter(_custom_field_data__cf9__contains="Foo"),
+        )
+        self.assertQuerysetEqual(
+            self.filterset({"cf_cf9": "Bar"}, self.queryset).qs,
+            self.queryset.filter(_custom_field_data__cf9__contains="Bar"),
+        )
 
     def test_filter_null_values(self):
-        self.assertEqual(self.filterset({"cf_cf8": "null"}, self.queryset).qs.count(), 2)
-        self.assertEqual(self.filterset({"cf_cf9": "null"}, self.queryset).qs.count(), 1)
+        self.assertQuerysetEqual(
+            self.filterset({"cf_cf8": "null"}, self.queryset).qs,
+            self.queryset.filter(_custom_field_data__cf8__isnull=True),
+        )
+        self.assertQuerysetEqual(
+            self.filterset({"cf_cf9": "null"}, self.queryset).qs,
+            self.queryset.filter(_custom_field_data__cf9__isnull=True),
+        )
 
 
 class CustomFieldChoiceTest(TestCase):
@@ -1537,6 +1872,49 @@ class CustomFieldChoiceTest(TestCase):
             self.assertEqual(CustomField.objects.count(), 0)
         self.assertEqual(CustomFieldChoice.objects.count(), 0)
 
+    @override_settings(
+        CELERY_TASK_ALWAYS_EAGER=True,
+        CELERY_TASK_EAGER_PROPOGATES=True,
+        CELERY_BROKER_URL="memory://",
+        CELERY_BACKEND="memory",
+    )
+    def test_regex_validation(self):
+        obj_type = ContentType.objects.get_for_model(Site)
+
+        for cf_type in CustomFieldTypeChoices.REGEX_TYPES:
+            # only validation for select and multi-select are performed on the CustomFieldChoice model
+            if "select" not in cf_type:
+                continue
+
+            # Create a custom field
+            cf = CustomField(
+                type=cf_type,
+                name=f"cf_test_{cf_type}",
+                required=False,
+                validation_regex="A.C[01]x?",
+            )
+            cf.save()
+            cf.content_types.set([obj_type])
+
+            non_matching_values = ["abc1", "AC1", "00AbC", "abc1x", "00abc1x00"]
+            for value in non_matching_values:
+                error_message = f"Value must match regex {cf.validation_regex} got {value}."
+                with self.subTest(cf_type=cf_type, value=value):
+                    with self.assertRaisesMessage(ValidationError, error_message):
+                        cfc = CustomFieldChoice.objects.create(field=cf, value=value)
+                        cfc.validated_save()
+
+            CustomFieldChoice.objects.all().delete()
+
+            matching_values = ["ABC1", "00AbC0", "00ABC0x00"]
+            for value in matching_values:
+                with self.subTest(cf_type=cf_type, value=value):
+                    cfc = CustomFieldChoice.objects.create(field=cf, value=value)
+                    cfc.validated_save()
+
+            # Delete the custom field
+            cf.delete()
+
 
 class CustomFieldBackgroundTasks(CeleryTestCase):
     def test_provision_field_task(self):
@@ -1568,6 +1946,7 @@ class CustomFieldBackgroundTasks(CeleryTestCase):
             type=CustomFieldTypeChoices.TYPE_TEXT,
         )
         cf.save()
+        logging.disable(logging.ERROR)
         cf.content_types.set([obj_type])
 
         site = Site(name="Site 1", slug="site-1", _custom_field_data={"cf1": "foo"})
@@ -1580,6 +1959,7 @@ class CustomFieldBackgroundTasks(CeleryTestCase):
         site.refresh_from_db()
 
         self.assertTrue("cf1" not in site.cf)
+        logging.disable(logging.NOTSET)
 
     def test_update_custom_field_choice_data_task(self):
         self.clear_worker()
