@@ -1,5 +1,6 @@
 import datetime
 import json
+import logging
 import re
 import uuid
 from io import StringIO
@@ -49,7 +50,7 @@ def create_job_result_and_run_job(module, name, *, data=None, commit=True, reque
     """Test helper function to call get_job_class_and_model() then and call run_job_for_testing()."""
     if data is None:
         data = {}
-    job_class, job_model = get_job_class_and_model(module, name)
+    _job_class, job_model = get_job_class_and_model(module, name)
     job_result = run_job_for_testing(job=job_model, data=data, commit=commit, request=request)
     job_result.refresh_from_db()
     return job_result
@@ -97,7 +98,7 @@ class JobTest(TransactionTestCase):
         """
         module = "test_pass"
         name = "TestPass"
-        job_class, job_model = get_job_class_and_model(module, name)
+        _job_class, job_model = get_job_class_and_model(module, name)
         job_model.enabled = True
         job_model.validated_save()
         job_content_type = ContentType.objects.get(app_label="extras", model="job")
@@ -127,7 +128,9 @@ class JobTest(TransactionTestCase):
         """
         module = "test_fail"
         name = "TestFail"
+        logging.disable(logging.ERROR)
         job_result = create_job_result_and_run_job(module, name, commit=False)
+        logging.disable(logging.NOTSET)
         self.assertEqual(job_result.status, JobResultStatusChoices.STATUS_ERRORED)
 
     def test_field_default(self):
@@ -140,17 +143,14 @@ class JobTest(TransactionTestCase):
         job_class = get_job(f"local/{module}/{name}")
         form = job_class().as_form()
 
-        self.assertHTMLEqual(
-            form.as_table(),
+        self.assertInHTML(
             """<tr><th><label for="id_var_int">Var int:</label></th><td>
 <input class="form-control form-control" id="id_var_int" max="3600" name="var_int" placeholder="None" required type="number" value="0">
 <br><span class="helptext">Test default of 0 Falsey</span></td></tr>
 <tr><th><label for="id_var_int_no_default">Var int no default:</label></th><td>
 <input class="form-control form-control" id="id_var_int_no_default" max="3600" name="var_int_no_default" placeholder="None" type="number">
-<br><span class="helptext">Test default without default</span></td></tr>
-<tr><th><label for="id__commit">Commit changes:</label></th><td>
-<input checked id="id__commit" name="_commit" placeholder="Commit changes" type="checkbox">
-<br><span class="helptext">Commit changes to the database (uncheck for a dry-run)</span></td></tr>""",
+<br><span class="helptext">Test default without default</span></td></tr>""",
+            form.as_table(),
         )
 
     def test_field_order(self):
@@ -161,7 +161,7 @@ class JobTest(TransactionTestCase):
         name = "TestFieldOrder"
         job_class = get_job(f"local/{module}/{name}")
         form = job_class().as_form()
-        self.assertSequenceEqual(list(form.fields.keys()), ["var1", "var2", "var23", "_commit"])
+        self.assertSequenceEqual(list(form.fields.keys()), ["var1", "var2", "var23", "_task_queue", "_commit"])
 
     def test_no_field_order(self):
         """
@@ -171,7 +171,7 @@ class JobTest(TransactionTestCase):
         name = "TestNoFieldOrder"
         job_class = get_job(f"local/{module}/{name}")
         form = job_class().as_form()
-        self.assertSequenceEqual(list(form.fields.keys()), ["var23", "var2", "_commit"])
+        self.assertSequenceEqual(list(form.fields.keys()), ["var23", "var2", "_task_queue", "_commit"])
 
     def test_no_field_order_inherited_variable(self):
         """
@@ -181,7 +181,10 @@ class JobTest(TransactionTestCase):
         name = "TestDefaultFieldOrderWithInheritance"
         job_class = get_job(f"local/{module}/{name}")
         form = job_class().as_form()
-        self.assertSequenceEqual(list(form.fields.keys()), ["testvar1", "b_testvar2", "a_testvar3", "_commit"])
+        self.assertSequenceEqual(
+            list(form.fields.keys()),
+            ["testvar1", "b_testvar2", "a_testvar3", "_task_queue", "_commit"],
+        )
 
     def test_read_only_job_pass(self):
         """
@@ -199,7 +202,9 @@ class JobTest(TransactionTestCase):
         """
         module = "test_read_only_fail"
         name = "TestReadOnlyFail"
+        logging.disable(logging.ERROR)
         job_result = create_job_result_and_run_job(module, name, commit=False)
+        logging.disable(logging.NOTSET)
         self.assertEqual(job_result.status, JobResultStatusChoices.STATUS_ERRORED)
         self.assertEqual(Site.objects.count(), 0)  # Ensure DB transaction was aborted
         # Also ensure the standard log message about aborting the transaction is *not* present
@@ -217,11 +222,9 @@ class JobTest(TransactionTestCase):
 
         form = job_class().as_form()
 
-        self.assertHTMLEqual(
+        self.assertInHTML(
+            "<input id='id__commit' name='_commit' type='hidden' value='False'>",
             form.as_table(),
-            """<tr><th><label for="id_var">Var:</label></th><td>
-<input class="form-control form-control" id="id_var" name="var" placeholder="None" required type="text">
-<br><span class="helptext">Hello</span><input id="id__commit" name="_commit" type="hidden" value="False"></td></tr>""",
         )
 
     def test_ip_address_vars(self):
@@ -236,7 +239,7 @@ class JobTest(TransactionTestCase):
         """
         module = "test_ipaddress_vars"
         name = "TestIPAddresses"
-        job_class, job_model = get_job_class_and_model(module, name)
+        job_class, _job_model = get_job_class_and_model(module, name)
 
         # Fill out the form
         form_data = dict(
@@ -336,8 +339,9 @@ class JobTest(TransactionTestCase):
         module = "test_object_var_required"
         name = "TestRequiredObjectVar"
         data = {"region": None}
-
+        logging.disable(logging.ERROR)
         job_result = create_job_result_and_run_job(module, name, data=data, commit=False)
+        logging.disable(logging.NOTSET)
 
         # Assert stuff
         self.assertEqual(job_result.status, JobResultStatusChoices.STATUS_ERRORED)
@@ -353,9 +357,9 @@ class JobTest(TransactionTestCase):
         module = "test_object_vars"
         name = "TestObjectVars"
         data = "BAD DATA STRING"
-
+        logging.disable(logging.ERROR)
         job_result = create_job_result_and_run_job(module, name, data=data, commit=False)
-
+        logging.disable(logging.NOTSET)
         # Assert stuff
         self.assertEqual(job_result.status, JobResultStatusChoices.STATUS_ERRORED)
         log_failure = JobLogEntry.objects.filter(
@@ -373,11 +377,58 @@ class JobTest(TransactionTestCase):
         self.assertEqual(job_result_1.status, JobResultStatusChoices.STATUS_COMPLETED)
         job_result_2 = create_job_result_and_run_job(module, name, commit=False)
         self.assertEqual(job_result_2.status, JobResultStatusChoices.STATUS_COMPLETED)
-        job_class, job_model = get_job_class_and_model(module, name)
+        _job_class, job_model = get_job_class_and_model(module, name)
         self.assertGreaterEqual(job_model.results.count(), 2)
         latest_job_result = job_model.latest_result
         self.assertEqual(job_result_2.completed, latest_job_result.completed)
 
+    @mock.patch("nautobot.extras.utils.get_celery_queues")
+    def test_job_class_task_queues(self, mock_get_celery_queues):
+        """
+        Test job form with custom task queues defined on the job class
+        """
+        module = "test_task_queues"
+        name = "TestWorkerQueues"
+        mock_get_celery_queues.return_value = {"celery": 4, "irrelevant": 5}
+        job_class, _ = get_job_class_and_model(module, name)
+        form = job_class().as_form()
+        self.assertInHTML(
+            """<tr><th><label for="id__task_queue">Task queue:</label></th>
+            <td><select name="_task_queue" class="form-control" placeholder="Task queue" id="id__task_queue">
+            <option value="celery">celery (4 workers)</option>
+            <option value="nonexistent">nonexistent (0 workers)</option></select><br>
+            <span class="helptext">The task queue to route this job to</span></td></tr>
+            <tr><th><label for="id__commit">Commit changes:</label></th>
+            <td><input type="checkbox" name="_commit" placeholder="Commit changes" id="id__commit" checked><br>
+            <span class="helptext">Commit changes to the database (uncheck for a dry-run)</span></td></tr>""",
+            form.as_table(),
+        )
+
+    @mock.patch("nautobot.extras.utils.get_celery_queues")
+    def test_job_class_task_queues_override(self, mock_get_celery_queues):
+        """
+        Test job form with custom task queues defined on the job class and overridden on the model
+        """
+        module = "test_task_queues"
+        name = "TestWorkerQueues"
+        mock_get_celery_queues.return_value = {"default": 1, "irrelevant": 5}
+        job_class, job_model = get_job_class_and_model(module, name)
+        job_model.task_queues = ["default", "priority"]
+        job_model.task_queues_override = True
+        job_model.save()
+        form = job_class().as_form()
+        self.assertInHTML(
+            """<tr><th><label for="id__task_queue">Task queue:</label></th>
+            <td><select name="_task_queue" class="form-control" placeholder="Task queue" id="id__task_queue">
+            <option value="default">default (1 worker)</option>
+            <option value="priority">priority (0 workers)</option>
+            </select><br><span class="helptext">The task queue to route this job to</span></td></tr>
+            <tr><th><label for="id__commit">Commit changes:</label></th>
+            <td><input type="checkbox" name="_commit" placeholder="Commit changes" id="id__commit" checked><br>
+            <span class="helptext">Commit changes to the database (uncheck for a dry-run)</span></td></tr>""",
+            form.as_table(),
+        )
+
 
 class JobFileUploadTest(TransactionTestCase):
     """Test a job that uploads/deletes files."""
@@ -399,7 +450,7 @@ class JobFileUploadTest(TransactionTestCase):
         """Test that file upload succeeds; job SUCCEEDS; and files are deleted."""
         module = "test_file_upload_pass"
         name = "TestFileUploadPass"
-        job_class, job_model = get_job_class_and_model(module, name)
+        job_class, _job_model = get_job_class_and_model(module, name)
 
         # Serialize the file to FileProxy
         data = {"file": self.test_file}
@@ -431,7 +482,7 @@ class JobFileUploadTest(TransactionTestCase):
         """Test that file upload succeeds; job FAILS; files deleted."""
         module = "test_file_upload_fail"
         name = "TestFileUploadFail"
-        job_class, job_model = get_job_class_and_model(module, name)
+        job_class, _job_model = get_job_class_and_model(module, name)
 
         # Serialize the file to FileProxy
         data = {"file": self.test_file}
@@ -445,7 +496,9 @@ class JobFileUploadTest(TransactionTestCase):
         self.assertEqual(FileProxy.objects.count(), 1)
 
         # Run the job
+        logging.disable(logging.ERROR)
         job_result = create_job_result_and_run_job(module, name, data=serialized_data, commit=False)
+        logging.disable(logging.NOTSET)
 
         # Assert that file contents were correctly read
         self.assertEqual(
@@ -485,7 +538,7 @@ class RunJobManagementCommandTest(CeleryTestCase):
         """Basic success-path test for Jobs that don't modify the Nautobot database."""
         module = "test_pass"
         name = "TestPass"
-        job_class, job_model = get_job_class_and_model(module, name)
+        _job_class, job_model = get_job_class_and_model(module, name)
 
         out, err = self.run_command(job_model.class_path)
         self.assertIn(f"Running {job_model.class_path}...", out)
@@ -502,7 +555,7 @@ class RunJobManagementCommandTest(CeleryTestCase):
 
         module = "test_modify_db"
         name = "TestModifyDB"
-        job_class, job_model = get_job_class_and_model(module, name)
+        _job_class, job_model = get_job_class_and_model(module, name)
 
         out, err = self.run_command(job_model.class_path)
         self.assertIn(f"Running {job_model.class_path}...", out)
@@ -522,7 +575,7 @@ class RunJobManagementCommandTest(CeleryTestCase):
         """A job that changes the DB, when run with commit=True but no username, is rejected."""
         module = "test_modify_db"
         name = "TestModifyDB"
-        job_class, job_model = get_job_class_and_model(module, name)
+        _job_class, job_model = get_job_class_and_model(module, name)
         with self.assertRaises(CommandError):
             self.run_command("--commit", job_model.class_path)
 
@@ -530,7 +583,7 @@ class RunJobManagementCommandTest(CeleryTestCase):
         """A job that changes the DB, when run with commit=True and a nonexistent username, is rejected."""
         module = "test_modify_db"
         name = "TestModifyDB"
-        job_class, job_model = get_job_class_and_model(module, name)
+        _job_class, job_model = get_job_class_and_model(module, name)
         with self.assertRaises(CommandError):
             self.run_command("--commit", "--username", "nosuchuser", job_model.class_path)
 
@@ -540,7 +593,7 @@ class RunJobManagementCommandTest(CeleryTestCase):
 
         module = "test_modify_db"
         name = "TestModifyDB"
-        job_class, job_model = get_job_class_and_model(module, name)
+        _job_class, job_model = get_job_class_and_model(module, name)
 
         out, err = self.run_command("--commit", "--username", "test_user", job_model.class_path)
         self.assertIn(f"Running {job_model.class_path}...", out)
@@ -593,6 +646,62 @@ class JobSiteCustomFieldTest(CeleryTestCase):
         self.assertEqual(site_2[0].cf["cf1"], "-")
 
 
+class JobButtonReceiverTest(TransactionTestCase):
+    """
+    Test job button receiver.
+    """
+
+    def setUp(self):
+        super().setUp()
+
+        # Initialize fake request that will be required to run jobs
+        self.request = RequestFactory().request(SERVER_NAME="WebRequestContext")
+        self.request.id = uuid.uuid4()
+        self.request.user = self.user
+
+        site = Site.objects.create(name="Job Button Test Site 1")
+        content_type = ContentType.objects.get_for_model(Site)
+        self.data = {
+            "object_pk": site.pk,
+            "object_model_name": f"{content_type.app_label}.{content_type.model}",
+        }
+
+    def test_form_field(self):
+        module = "test_job_button_receiver"
+        name = "TestJobButtonReceiverSimple"
+        job_class, _job_model = get_job_class_and_model(module, name)
+        form = job_class().as_form()
+        self.assertSequenceEqual(list(form.fields.keys()), ["object_pk", "object_model_name", "_task_queue", "_commit"])
+
+    def test_hidden(self):
+        module = "test_job_button_receiver"
+        name = "TestJobButtonReceiverSimple"
+        _job_class, job_model = get_job_class_and_model(module, name)
+        self.assertFalse(job_model.hidden)
+
+    def test_is_job_button(self):
+
+        with self.subTest(expected=False):
+            module = "test_pass"
+            name = "TestPass"
+            _job_class, job_model = get_job_class_and_model(module, name)
+            self.assertFalse(job_model.is_job_button_receiver)
+
+        with self.subTest(expected=True):
+            module = "test_job_button_receiver"
+            name = "TestJobButtonReceiverSimple"
+            _job_class, job_model = get_job_class_and_model(module, name)
+            self.assertTrue(job_model.is_job_button_receiver)
+
+    def test_missing_receive_job_button_method(self):
+        module = "test_job_button_receiver"
+        name = "TestJobButtonReceiverFail"
+        logging.disable(logging.ERROR)
+        job_result = create_job_result_and_run_job(module, name, data=self.data, commit=False)
+        logging.disable(logging.NOTSET)
+        self.assertEqual(job_result.status, JobResultStatusChoices.STATUS_ERRORED)
+
+
 class JobHookReceiverTest(TransactionTestCase):
     """
     Test job hook receiver.
@@ -617,14 +726,14 @@ class JobHookReceiverTest(TransactionTestCase):
     def test_form_field(self):
         module = "test_job_hook_receiver"
         name = "TestJobHookReceiverLog"
-        job_class, job_model = get_job_class_and_model(module, name)
+        job_class, _job_model = get_job_class_and_model(module, name)
         form = job_class().as_form()
-        self.assertCountEqual(form.fields.keys(), ["object_change", "_commit"])
+        self.assertSequenceEqual(list(form.fields.keys()), ["object_change", "_task_queue", "_commit"])
 
     def test_hidden(self):
         module = "test_job_hook_receiver"
         name = "TestJobHookReceiverLog"
-        job_class, job_model = get_job_class_and_model(module, name)
+        _job_class, job_model = get_job_class_and_model(module, name)
         self.assertFalse(job_model.hidden)
 
     def test_is_job_hook(self):
@@ -632,13 +741,13 @@ class JobHookReceiverTest(TransactionTestCase):
         with self.subTest(expected=False):
             module = "test_pass"
             name = "TestPass"
-            job_class, job_model = get_job_class_and_model(module, name)
+            _job_class, job_model = get_job_class_and_model(module, name)
             self.assertFalse(job_model.is_job_hook_receiver)
 
         with self.subTest(expected=True):
             module = "test_job_hook_receiver"
             name = "TestJobHookReceiverLog"
-            job_class, job_model = get_job_class_and_model(module, name)
+            _job_class, job_model = get_job_class_and_model(module, name)
             self.assertTrue(job_model.is_job_hook_receiver)
 
     def test_object_change_context(self):
@@ -653,7 +762,9 @@ class JobHookReceiverTest(TransactionTestCase):
     def test_missing_receive_job_hook_method(self):
         module = "test_job_hook_receiver"
         name = "TestJobHookReceiverFail"
+        logging.disable(logging.ERROR)
         job_result = create_job_result_and_run_job(module, name, data=self.data, commit=False)
+        logging.disable(logging.NOTSET)
         self.assertEqual(job_result.status, JobResultStatusChoices.STATUS_ERRORED)
 
 
@@ -693,12 +804,12 @@ class JobHookTest(CeleryTestCase):
             self.assertSequenceEqual(log_messages, expected_log_messages)
 
     @mock.patch.object(JobResult, "enqueue_job")
-    def test_enqueue_job_hook_skipped(self, mock):
+    def test_enqueue_job_hook_skipped(self, mock_enqueue_job):
         change_context = JobHookChangeContext(user=self.user)
         with change_logging(change_context):
             Site.objects.create(name="Test Job Hook Site 2")
 
-        self.assertFalse(mock.called)
+        self.assertFalse(mock_enqueue_job.called)
 
 
 class RemoveScheduledJobManagementCommandTestCase(TestCase):
@@ -731,3 +842,28 @@ class RemoveScheduledJobManagementCommandTestCase(TestCase):
         self.assertTrue(ScheduledJob.objects.filter(name="test1").exists())
         for i in range(2, 7):
             self.assertFalse(ScheduledJob.objects.filter(name=f"test{i}").exists())
+
+
+class ScheduledJobIntervalTestCase(TestCase):
+    """Test scheduled job intervals"""
+
+    # cron schedule day_of_week starts on Sunday (Sunday = 0)
+    cron_days = ["Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"]
+    # datetime weekday starts on Monday (Sunday = 6)
+    datetime_days = ["Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday"]
+
+    def test_weekly_interval(self):
+        start_time = timezone.now() + datetime.timedelta(days=6)
+        scheduled_job = ScheduledJob.objects.create(
+            name="weekly_interval",
+            task="nautobot.extras.jobs.scheduled_job_handler",
+            job_class="local/test_pass/TestPass",
+            interval=JobExecutionType.TYPE_WEEKLY,
+            user=self.user,
+            start_time=start_time,
+        )
+
+        requested_weekday = self.datetime_days[start_time.weekday()]
+        schedule_day_of_week = list(scheduled_job.schedule.day_of_week)[0]
+        scheduled_job_weekday = self.cron_days[schedule_day_of_week]
+        self.assertEqual(scheduled_job_weekday, requested_weekday)
