@@ -12,10 +12,11 @@ import django_filters
 from django import forms
 from django.conf import settings
 from django.contrib.auth import get_user_model
+from django.contrib.auth.models import Group
 from django.contrib.contenttypes.models import ContentType
 from django.core.exceptions import ValidationError
 from django.core.serializers import serialize
-from django.db.models import Count, ForeignKey, Model, OuterRef, Subquery
+from django.db.models import Count, Model, OuterRef, Subquery
 from django.db.models.functions import Coalesce
 from django.http import QueryDict
 from django.template import engines
@@ -105,7 +106,15 @@ def get_route_for_model(model, action, api=False):
         model = get_model_from_name(model)
 
     suffix = "" if not api else "-api"
-    prefix = f"{model._meta.app_label}{suffix}:{model._meta.model_name}"
+    # The `contenttypes` and `auth` app doesn't provide REST API endpoints,
+    # but Nautobot provides one for the ContentType model in our `extras` and Group model in `users` app.
+    if model is ContentType:
+        app_label = "extras"
+    elif model is Group:
+        app_label = "users"
+    else:
+        app_label = model._meta.app_label
+    prefix = f"{app_label}{suffix}:{model._meta.model_name}"
     sep = "_" if not api else "-"
     viewname = f"{prefix}{sep}{action}"
 
@@ -280,7 +289,7 @@ def dict_to_filter_params(d, prefix=""):
     return params
 
 
-def normalize_querydict(querydict):
+def normalize_querydict(querydict, form_class=None):
     """
     Convert a QueryDict to a normal, mutable dictionary, preserving list values. For example,
 
@@ -292,10 +301,28 @@ def normalize_querydict(querydict):
 
     This function is necessary because QueryDict does not provide any built-in mechanism which preserves multiple
     values.
+
+    A `form_class` can be provided as a way to hint which query parameters should be treated as lists.
     """
-    if not querydict:
-        return {}
-    return {k: v if len(v) > 1 else v[0] for k, v in querydict.lists()}
+    result = {}
+    if querydict:
+        for key, value_list in querydict.lists():
+            if len(value_list) > 1:
+                # More than one value in the querydict for this key, so keep it as a list
+                # TODO: we could check here and de-listify value_list if the form_class field is a single-value one?
+                result[key] = value_list
+            elif (
+                form_class is not None
+                and key in form_class.base_fields
+                # ModelMultipleChoiceField is *not* itself a subclass of MultipleChoiceField, thanks Django!
+                and isinstance(form_class.base_fields[key], (forms.MultipleChoiceField, forms.ModelMultipleChoiceField))
+            ):
+                # Even though there's only a single value in the querydict for this key, the form wants it as a list
+                result[key] = value_list
+            else:
+                # Only a single value in the querydict for this key, and no guidance otherwise, so make it single
+                result[key] = value_list[0]
+    return result
 
 
 def deepmerge(original, new):
@@ -323,15 +350,19 @@ def to_meters(length, unit):
     if unit not in valid_units:
         raise ValueError(f"Unknown unit {unit}. Must be one of the following: {', '.join(valid_units)}")
 
+    if unit == CableLengthUnitChoices.UNIT_KILOMETER:
+        return length * 1000
     if unit == CableLengthUnitChoices.UNIT_METER:
         return length
     if unit == CableLengthUnitChoices.UNIT_CENTIMETER:
         return length / 100
+    if unit == CableLengthUnitChoices.UNIT_MILE:
+        return length * Decimal("1609.344")
     if unit == CableLengthUnitChoices.UNIT_FOOT:
         return length * Decimal("0.3048")
     if unit == CableLengthUnitChoices.UNIT_INCH:
         return length * Decimal("0.3048") * 12
-    raise ValueError(f"Unknown unit {unit}. Must be 'm', 'cm', 'ft', or 'in'.")
+    raise ValueError(f"Unknown unit {unit}. Must be 'km', 'm', 'cm', 'mi', 'ft', or 'in'.")
 
 
 def render_jinja2(template_code, context):
@@ -348,27 +379,11 @@ def prepare_cloned_fields(instance):
     Compile an object's `clone_fields` list into a string of URL query parameters. Tags are automatically cloned where
     applicable.
     """
-    form_class = get_form_for_model(instance)
-    form = form_class() if form_class is not None else None
     params = []
     for field_name in getattr(instance, "clone_fields", []):
         field = instance._meta.get_field(field_name)
         field_value = field.value_from_object(instance)
 
-        # For foreign-key fields, if the ModelForm's field has a defined `to_field_name`,
-        # use that field from the related object instead of its PK.
-        # Example: Location.parent, LocationForm().fields["parent"].to_field_name = "slug", so use slug rather than PK.
-        if isinstance(field, ForeignKey):
-            related_object = getattr(instance, field_name)
-            if (
-                related_object is not None
-                and form is not None
-                and field_name in form.fields
-                and hasattr(form.fields[field_name], "to_field_name")
-                and form.fields[field_name].to_field_name is not None
-            ):
-                field_value = getattr(related_object, form.fields[field_name].to_field_name)
-
         # Swap out False with URL-friendly value
         if field_value is False:
             field_value = ""
@@ -496,12 +511,10 @@ class NautobotFakeRequest:
         data = copy.deepcopy(self.__dict__)
         # We don't want to try to pickle/unpickle or serialize/deserialize the actual User object,
         # but make sure we do store its PK so that we can look it up on-demand after being deserialized.
+        user = data.pop("user")
+        data.pop("_cached_user", None)
         if "_user_pk" not in data:
-            # We have a user but haven't stored its PK yet, so look it up and store it
-            data["_user_pk"] = data.pop("user").pk
-        else:
-            # We already have stored the PK, and we need to AVOID actually resolving the lazy user reference.
-            data.pop("user")
+            data["_user_pk"] = user.pk
         return data
 
     @classmethod
@@ -847,7 +860,7 @@ def get_filterset_parameter_form_field(model, parameter):
     Return the relevant form field instance for a filterset parameter e.g DynamicModelMultipleChoiceField, forms.IntegerField e.t.c
     """
     # Avoid circular import
-    from nautobot.extras.filters import ContentTypeMultipleChoiceFilter, StatusFilter
+    from nautobot.extras.filters import ContentTypeMultipleChoiceFilter, CustomFieldFilterMixin, StatusFilter
     from nautobot.extras.models import Status, Tag
     from nautobot.extras.utils import ChangeLoggedModelsQuery, TaggableClassesQuery
     from nautobot.utilities.forms import (
@@ -869,7 +882,9 @@ def get_filterset_parameter_form_field(model, parameter):
     form_field = field.field
 
     # TODO(Culver): We are having to replace some widgets here because multivalue_field_factory that generates these isn't smart enough
-    if isinstance(field, NumberFilter):
+    if isinstance(field, CustomFieldFilterMixin):
+        form_field = field.custom_field.to_form_field()
+    elif isinstance(field, NumberFilter):
         form_field = forms.IntegerField()
     elif isinstance(field, filters.ModelMultipleChoiceFilter):
         related_model = Status if isinstance(field, StatusFilter) else field.extra["queryset"].model
