@@ -208,7 +208,7 @@ class ClusterAddDevicesView(generic.ObjectEditView):
 
     def get(self, request, *args, **kwargs):
         cluster = get_object_or_404(self.queryset, pk=kwargs["pk"])
-        form = self.form(cluster, initial=normalize_querydict(request.GET))
+        form = self.form(cluster, initial=normalize_querydict(request.GET, form_class=self.form))
 
         return render(
             request,
@@ -455,6 +455,12 @@ class VMInterfaceBulkRenameView(generic.BulkRenameView):
     queryset = VMInterface.objects.all()
     form = forms.VMInterfaceBulkRenameForm
 
+    def get_selected_objects_parents_name(self, selected_objects):
+        selected_object = selected_objects.first()
+        if selected_object:
+            return selected_object.virtual_machine.name
+        return None
+
 
 class VMInterfaceBulkDeleteView(generic.BulkDeleteView):
     queryset = VMInterface.objects.all()
